<?php

namespace Drupal\smtp_multiple\Plugin\Mail;

use Drupal\smtp\Plugin\Mail\SMTPMailSystem;

/**
 * Allow for sending SMTP messages with multiple configurations.
 *
 * @Mail(
 *   id = "smtp_multiple_mail_system",
 *   label = @Translation("SMTP Multiple Mailer"),
 *   description = @Translation("Allows for sending SMTP messages with multiple configurations.")
 * )
 *
 * Note: We don't use dependency injection here in order for the parent's
 * constructor changes not to break this here module too easily.
 * @see https://www.drupal.org/project/smtp_multiple/issues/3143194.
 */
class SmtpMultipleMailSystem extends SMTPMailSystem {

  public function mail(array $message) {
    if (empty($message['id'])) {
      return parent::mail($message);
    }

    $message_id = $message['id'];

    $original_config = $config = $this->smtpConfig->get();
    unset($config['langcode'], $config['_core']);

    // Alter config in hooks.
    \Drupal::moduleHandler()->alter('smtp_multiple_config', $config, $message_id);

    // Alter config in settings file.
    $config_overrides = \Drupal::service('settings')->get('smtp_multiple_config');
    if (!empty($config_overrides[$message_id])) {
      foreach ($config_overrides[$message_id] as $key => $override) {
        $config[$key] = $override;
      }
    }

    $this->smtpConfig->setSettingsOverride($config);
    $result = parent::mail($message);
    $this->smtpConfig->setSettingsOverride($original_config);

    return $result;
  }

}
